"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceMessageHandler = void 0;
const error_1 = require("../error");
const command_1 = require("./command");
const event_1 = require("./event");
class DeviceMessageHandler {
    static getStreamingDevices(stationSN) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined)
            return DeviceMessageHandler.streamingDevices[stationSN];
        return [];
    }
    static removeStreamingDevice(stationSN, client) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined)
            DeviceMessageHandler.streamingDevices[stationSN] = DeviceMessageHandler.streamingDevices[stationSN].filter((cl) => cl !== client);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static async handle(message, driver, client) {
        const { serialNumber, command } = message;
        const device = driver.getDevice(serialNumber);
        const station = driver.getStation(device.getStationSerial());
        switch (command) {
            case command_1.DeviceCommand.setStatusLed:
                await station.setStatusLed(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setAutoNightVision:
                await station.setAutoNightVision(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setMotionDetection:
                await station.setMotionDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setSoundDetection:
                await station.setSoundDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setPetDetection:
                await station.setPetDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setRTSPStream:
                await station.setRTSPStream(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setAntiTheftDetection:
                await station.setAntiTheftDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setWatermark:
                await station.setWatermark(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.enableDevice:
                await station.enableDevice(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.lockDevice:
                await station.lockDevice(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.getPropertiesMetadata:
                {
                    const properties = device.getPropertiesMetadata();
                    return { properties: properties };
                }
            case command_1.DeviceCommand.getProperties:
                {
                    const properties = device.getProperties();
                    return { properties: properties };
                }
            case command_1.DeviceCommand.setProperty:
                await driver.setDeviceProperty(serialNumber, message.name, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.startLivestream:
                if (!station.isLiveStreaming(device)) {
                    await station.startLivestream(device).catch((error) => {
                        throw error;
                    });
                    client.receiveLivestream[serialNumber] = true;
                    DeviceMessageHandler.streamingDevices[station.getSerial()] = [client];
                }
                else if (client.receiveLivestream[serialNumber] !== true) {
                    //TODO: Cache last received I-Frame and send that with the rest to the new client
                    client.sendEvent({
                        source: "device",
                        event: event_1.DeviceEvent.livestreamStarted,
                        serialNumber: serialNumber,
                    });
                    client.receiveLivestream[serialNumber] = true;
                    if (DeviceMessageHandler.streamingDevices[station.getSerial()] !== undefined)
                        DeviceMessageHandler.streamingDevices[station.getSerial()].push(client);
                }
                else {
                    throw new error_1.LivestreamAlreadyRunningError(`Livestream for device ${serialNumber} is already running`);
                }
                return {};
            case command_1.DeviceCommand.stopLivestream:
                if (client.receiveLivestream[serialNumber] !== true) {
                    throw new error_1.LivestreamNotRunningError(`Start of livestream for device ${serialNumber} was not also requested by this client`);
                }
                if (DeviceMessageHandler.streamingDevices[station.getSerial()] !== undefined && DeviceMessageHandler.streamingDevices[station.getSerial()].includes(client)) {
                    if (DeviceMessageHandler.streamingDevices[station.getSerial()].length === 1) {
                        await station.stopLivestream(device).catch((error) => {
                            throw error;
                        });
                        delete DeviceMessageHandler.streamingDevices[station.getSerial()];
                    }
                    else {
                        client.receiveLivestream[serialNumber] = false;
                        DeviceMessageHandler.streamingDevices[station.getSerial()] = DeviceMessageHandler.streamingDevices[station.getSerial()].filter((cl) => cl !== client);
                        client.sendEvent({
                            source: "device",
                            event: event_1.DeviceEvent.livestreamStopped,
                            serialNumber: serialNumber,
                        });
                    }
                }
                return {};
            case command_1.DeviceCommand.isLiveStreaming:
                {
                    const result = station.isLiveStreaming(device);
                    return { livestreaming: result };
                }
            case command_1.DeviceCommand.triggerAlarm:
                await station.triggerDeviceAlarmSound(device, message.seconds).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.resetAlarm:
                await station.resetDeviceAlarmSound(device).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.panAndTilt:
                await station.panAndTilt(device, message.direction).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.quickResponse:
                await station.quickResponse(device, message.voiceId).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.startDownload:
                await station.startDownload(device, message.path, message.cipherId).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.cancelDownload:
                await station.cancelDownload(device).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.getVoices:
                const voices = await driver.getApi().getVoices(device.getSerial()).catch((error) => {
                    throw error;
                });
                return { voices: voices };
            default:
                throw new error_1.UnknownCommandError(command);
        }
    }
}
exports.DeviceMessageHandler = DeviceMessageHandler;
DeviceMessageHandler.streamingDevices = {};
//# sourceMappingURL=message_handler.js.map